@@ -1,4 +1,5 @@
-/* Licensed to the Apache Software Foundation (ASF) under one
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
  * regarding copyright ownership.  The ASF licenses this file
@@ -83,6 +84,8 @@ pub enum IggyError {
     InvalidClientAddress = 34,
     #[error("Invalid IP address: {0}:{1}")]
     InvalidIpAddress(String, String) = 35,
+    #[error("Http error {0}")]
+    HttpError(String) = 36,
     #[error("Unauthenticated")]
     Unauthenticated = 40,
     #[error("Unauthorized")]
@@ -404,6 +407,8 @@ pub enum IggyError {
     ProducerClosed = 4057,
     #[error("Invalid offset: {0}")]
     InvalidOffset(u64) = 4100,
+    #[error("Invalid reserved field value: {0}, expected: 0")]
+    InvalidReservedField(u64) = 4101,
     #[error("Consumer group with ID: {0} for topic with ID: {1} was not found.")]
     ConsumerGroupIdNotFound(Identifier, Identifier) = 5000,
     #[error("Invalid consumer group ID")]
