@@ -15,8 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::types::consensus::header::{
-    self, CommitHeader, ConsensusHeader, GenericHeader, PrepareHeader, ReplyHeader,
+use crate::{
+    header::RequestHeader,
+    types::consensus::header::{self, ConsensusHeader, PrepareHeader, PrepareOkHeader},
 };
 use bytes::Bytes;
 use std::marker::PhantomData;
@@ -89,24 +90,25 @@ where
         }
     }
 
-    /// Replace the header with a new one, using the provided function to generate it.
-    pub fn replace_header<T: ConsensusHeader>(self, f: impl FnOnce(&H) -> T) -> Message<T> {
+    /// Transmute the header to a different type, using the provided function to modify the new header.
+    pub fn transmute_header<T: ConsensusHeader>(self, f: impl FnOnce(H, &mut T)) -> Message<T> {
         assert_eq!(size_of::<H>(), size_of::<T>());
-        let prev = self.header();
-        let header = f(prev);
 
-        let header_bytes = bytemuck::bytes_of(&header);
-        let buffer = self.into_inner();
+        // Copy old header to stack to avoid UB.
+        let old_header = *self.header();
+
         // Safety: We ensured that size_of::<H>() == size_of::<T>()
         // On top of that, there is going to be only one reference to buffer during this function call
         // so no other references can observe the mutation.
         // In the future we can replace the `Bytes` buffer with something that does not allow sharing between different threads.
+        let buffer = self.into_inner();
         unsafe {
             let ptr = buffer.as_ptr() as *mut u8;
-            let slice = std::slice::from_raw_parts_mut(ptr, buffer.len());
-            slice[..size_of::<H>()].copy_from_slice(header_bytes);
+            let slice = std::slice::from_raw_parts_mut(ptr, size_of::<T>());
+            let new_header = bytemuck::from_bytes_mut(slice);
+            f(old_header, new_header);
         }
-        // TODO: Recalculate checksums
+
         Message {
             buffer,
             _marker: PhantomData,
@@ -140,6 +142,22 @@ where
         }
     }
 
+    /// Get the message body as zero-copy `Bytes`.
+    ///
+    /// Returns an empty `Bytes` if there is no body.
+    #[inline]
+    #[allow(unused)]
+    pub fn body_bytes(&self) -> Bytes {
+        let header_size = size_of::<H>();
+        let total_size = self.header().size() as usize;
+
+        if total_size > header_size {
+            self.buffer.slice(header_size..total_size)
+        } else {
+            Bytes::new()
+        }
+    }
+
     /// Get the complete message as bytes (header + body).
     #[inline]
     #[allow(unused)]
@@ -261,30 +279,36 @@ where
 #[derive(Debug)]
 #[allow(unused)]
 pub enum MessageBag {
-    Generic(Message<GenericHeader>),
+    Request(Message<RequestHeader>),
     Prepare(Message<PrepareHeader>),
-    Commit(Message<CommitHeader>),
-    Reply(Message<ReplyHeader>),
+    PrepareOk(Message<PrepareOkHeader>),
 }
 
 impl MessageBag {
     #[allow(unused)]
     pub fn command(&self) -> header::Command2 {
         match self {
-            MessageBag::Generic(message) => message.header().command,
+            MessageBag::Request(message) => message.header().command,
             MessageBag::Prepare(message) => message.header().command,
-            MessageBag::Commit(message) => message.header().command,
-            MessageBag::Reply(message) => message.header().command,
+            MessageBag::PrepareOk(message) => message.header().command,
         }
     }
 
     #[allow(unused)]
     pub fn size(&self) -> u32 {
         match self {
-            MessageBag::Generic(message) => message.header().size(),
+            MessageBag::Request(message) => message.header().size(),
             MessageBag::Prepare(message) => message.header().size(),
-            MessageBag::Commit(message) => message.header().size(),
-            MessageBag::Reply(message) => message.header().size(),
+            MessageBag::PrepareOk(message) => message.header().size(),
+        }
+    }
+
+    #[allow(unused)]
+    pub fn operation(&self) -> header::Operation {
+        match self {
+            MessageBag::Request(message) => message.header().operation,
+            MessageBag::Prepare(message) => message.header().operation,
+            MessageBag::PrepareOk(message) => message.header().operation,
         }
     }
 }
@@ -306,17 +330,17 @@ where
                     unsafe { Message::<header::PrepareHeader>::from_buffer_unchecked(buffer) };
                 MessageBag::Prepare(msg)
             }
-            header::Command2::Commit => {
-                let msg = unsafe { Message::<header::CommitHeader>::from_buffer_unchecked(buffer) };
-                MessageBag::Commit(msg)
+            header::Command2::Request => {
+                let msg =
+                    unsafe { Message::<header::RequestHeader>::from_buffer_unchecked(buffer) };
+                MessageBag::Request(msg)
             }
-            header::Command2::Reply => {
-                let msg = unsafe { Message::<header::ReplyHeader>::from_buffer_unchecked(buffer) };
-                MessageBag::Reply(msg)
+            header::Command2::PrepareOk => {
+                let msg =
+                    unsafe { Message::<header::PrepareOkHeader>::from_buffer_unchecked(buffer) };
+                MessageBag::PrepareOk(msg)
             }
-            _ => unreachable!(
-                "For now we only support Prepare, Commit, and Reply. In the future we will support more commands. Command2: {command:?}"
-            ),
+            _ => unreachable!(),
         }
     }
 }
@@ -480,33 +504,6 @@ mod tests {
 
         assert_eq!(bag.command(), header::Command2::Prepare);
         assert!(matches!(bag, MessageBag::Prepare(_)));
-        assert!(!matches!(bag, MessageBag::Commit(_)));
-        assert!(!matches!(bag, MessageBag::Reply(_)));
-        assert!(!matches!(bag, MessageBag::Generic(_)));
-    }
-
-    #[test]
-    fn test_message_bag_from_commit() {
-        let commit = header::CommitHeader::create_test();
-        let bag = MessageBag::from(commit);
-
-        assert_eq!(bag.command(), header::Command2::Commit);
-        assert!(!matches!(bag, MessageBag::Prepare(_)));
-        assert!(matches!(bag, MessageBag::Commit(_)));
-        assert!(!matches!(bag, MessageBag::Reply(_)));
-        assert!(!matches!(bag, MessageBag::Generic(_)));
-    }
-
-    #[test]
-    fn test_message_bag_from_reply() {
-        let reply = header::ReplyHeader::create_test();
-        let bag = MessageBag::from(reply);
-
-        assert_eq!(bag.command(), header::Command2::Reply);
-        assert!(!matches!(bag, MessageBag::Prepare(_)));
-        assert!(!matches!(bag, MessageBag::Commit(_)));
-        assert!(matches!(bag, MessageBag::Reply(_)));
-        assert!(!matches!(bag, MessageBag::Generic(_)));
     }
 }
 
