@@ -29,8 +29,12 @@ pub(crate) mod identifier;
 pub(crate) mod message;
 pub(crate) mod partition;
 pub(crate) mod permissions;
+pub(crate) mod personal_access_tokens;
+pub(crate) mod segment;
+pub(crate) mod segment_storage;
 pub(crate) mod snapshot;
 pub(crate) mod stats;
 pub(crate) mod stream;
+pub(crate) mod streaming_stats;
 pub(crate) mod topic;
 pub(crate) mod user;
