@@ -22,12 +22,12 @@ use crate::configs::connectors::{
     ConnectorsConfigProvider, CreateSinkConfig, CreateSourceConfig, SinkConfig, SourceConfig,
 };
 use crate::error::RuntimeError;
+use ::configs::{ConfigProvider, FileConfigProvider, TypedEnvProvider};
 use async_trait::async_trait;
-use chrono::{DateTime, Utc};
 use dashmap::DashMap;
 use figment::value::Dict;
 use figment::{Metadata, Profile, Provider};
-use iggy_common::{ConfigProvider, CustomEnvProvider, FileConfigProvider};
+use iggy_common::{DateTime, Utc};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::path::Path;
@@ -164,35 +164,77 @@ impl LocalConnectorsConfigProvider<Created> {
 
         let sinks: DashMap<ConnectorId, SinkConfigFile> = DashMap::new();
         let sources: DashMap<ConnectorId, SourceConfigFile> = DashMap::new();
-        info!("Loading connectors configuration from: {}", self.config_dir);
+        let cwd = match std::env::current_dir() {
+            Ok(path) => path.display().to_string(),
+            Err(_) => "unknown".to_string(),
+        };
+        info!(
+            "Loading connectors configuration from: {}, current directory: {cwd}",
+            self.config_dir
+        );
         let entries = std::fs::read_dir(&self.config_dir)?;
         for entry in entries.flatten() {
             let path = entry.path();
             if path.is_file() {
-                if let Some(file_name) = path.file_name().and_then(|n| n.to_str())
-                    && file_name.starts_with('.')
-                {
-                    debug!("Skipping hidden file: {:?}", path);
+                if path.extension().and_then(|ext| ext.to_str()) != Some("toml") {
+                    debug!("Skipping non-TOML file: {}", path.display());
                     continue;
                 }
 
-                debug!("Loading connector configuration from: {:?}", path);
+                if let Some(file_name) = path.file_name().and_then(|n| n.to_str()) {
+                    if file_name.starts_with('.') {
+                        debug!("Skipping hidden file: {}", path.display());
+                        continue;
+                    }
+                    let file_name_lower = file_name.to_lowercase();
+                    if file_name_lower == "cargo.toml" {
+                        debug!("Skipping Cargo.toml: {}", path.display());
+                        continue;
+                    }
+                }
+
+                info!("Loading connector configuration from: {}", path.display());
                 let base_config = Self::read_base_config(&path)?;
                 debug!("Loaded base configuration: {:?}", base_config);
                 let path = path
                     .to_str()
-                    .expect("Failed to convert connector configuration path to string")
+                    .ok_or_else(|| {
+                        RuntimeError::InvalidConfiguration(format!(
+                            "Non-UTF8 connector config path: {}",
+                            path.display()
+                        ))
+                    })?
                     .to_string();
                 let connector_config: ConnectorConfig =
                     Self::create_file_config_provider(path.clone(), &base_config)
                         .load_config()
                         .await
-                        .expect("Failed to load connector configuration");
+                        .map_err(|e| {
+                            RuntimeError::InvalidConfiguration(format!(
+                                "Failed to load connector configuration from '{path}': {e}"
+                            ))
+                        })?;
 
-                let created_at: DateTime<Utc> = entry.metadata()?.created()?.into();
+                let metadata = entry.metadata()?;
+                let created_at: DateTime<Utc> = metadata
+                    .created()
+                    .or_else(|_| metadata.modified())
+                    .map(Into::into)
+                    .unwrap_or_else(|_| {
+                        warn!(
+                            "Could not read created or modified time for '{path}', using current time",
+                        );
+                        Utc::now()
+                    });
                 let connector_id: ConnectorId = (&connector_config).into();
-                match connector_config.clone() {
-                    ConnectorConfig::Sink(sink_config) => {
+                let version = connector_config.version();
+
+                match connector_config {
+                    ConnectorConfig::Sink(mut sink_config) => {
+                        Self::apply_plugin_config_env_overrides(
+                            &mut sink_config.plugin_config,
+                            &base_config,
+                        );
                         sinks.insert(
                             connector_id,
                             SinkConfigFile {
@@ -202,7 +244,11 @@ impl LocalConnectorsConfigProvider<Created> {
                             },
                         );
                     }
-                    ConnectorConfig::Source(source_config) => {
+                    ConnectorConfig::Source(mut source_config) => {
+                        Self::apply_plugin_config_env_overrides(
+                            &mut source_config.plugin_config,
+                            &base_config,
+                        );
                         sources.insert(
                             connector_id,
                             SourceConfigFile {
@@ -214,10 +260,10 @@ impl LocalConnectorsConfigProvider<Created> {
                     }
                 }
 
-                debug!(
-                    "Loaded connector configuration with key {}, version {}, created at {}",
+                info!(
+                    "Loaded connector configuration with key: {}, version: {}, created at {}",
                     base_config.key(),
-                    connector_config.version(),
+                    version,
                     created_at.to_rfc3339()
                 );
             }
@@ -294,6 +340,31 @@ impl<S: ProviderState> LocalConnectorsConfigProvider<S> {
             ))
         })
     }
+
+    fn apply_plugin_config_env_overrides(
+        plugin_config: &mut Option<serde_json::Value>,
+        base_config: &BaseConnectorConfig,
+    ) {
+        let connector_type = base_config.connector_type().to_uppercase();
+        let key = base_config.key().to_uppercase();
+        let prefix = format!("IGGY_CONNECTORS_{connector_type}_{key}_PLUGIN_CONFIG_");
+
+        for (env_key, env_value) in std::env::vars() {
+            let env_key_upper = env_key.to_uppercase();
+            if !env_key_upper.starts_with(&prefix) {
+                continue;
+            }
+
+            let field_path = &env_key_upper[prefix.len()..];
+            let field_name = field_path.to_lowercase();
+            let parsed_value = ::configs::parse_env_value_to_json(&env_value);
+
+            let config = plugin_config.get_or_insert_with(|| serde_json::json!({}));
+            if let serde_json::Value::Object(map) = config {
+                map.insert(field_name, parsed_value);
+            }
+        }
+    }
 }
 
 #[derive(Debug, Deserialize)]
@@ -731,9 +802,15 @@ impl ConnectorsConfigProvider for LocalConnectorsConfigProvider<Initialized> {
 }
 
 #[derive(Debug, Clone)]
-pub struct ConnectorEnvProvider {
-    connector_name: String,
-    provider: CustomEnvProvider<ConnectorConfig>,
+pub enum ConnectorEnvProvider {
+    Sink {
+        connector_name: String,
+        provider: TypedEnvProvider<SinkConfig>,
+    },
+    Source {
+        connector_name: String,
+        provider: TypedEnvProvider<SourceConfig>,
+    },
 }
 
 impl ConnectorEnvProvider {
@@ -741,24 +818,38 @@ impl ConnectorEnvProvider {
         let connector_type = base_config.connector_type().to_uppercase();
         let key = base_config.key().to_uppercase();
         let prefix = format!("IGGY_CONNECTORS_{}_{}_", connector_type, key);
-        Self {
-            connector_name: base_config.key().to_owned(),
-            provider: CustomEnvProvider::new(&prefix, &[]),
+        let connector_name = base_config.key().to_owned();
+
+        match base_config {
+            BaseConnectorConfig::Sink { .. } => Self::Sink {
+                connector_name,
+                provider: TypedEnvProvider::with_runtime_prefix(&prefix, &[]),
+            },
+            BaseConnectorConfig::Source { .. } => Self::Source {
+                connector_name,
+                provider: TypedEnvProvider::with_runtime_prefix(&prefix, &[]),
+            },
         }
     }
 }
 
 impl Provider for ConnectorEnvProvider {
     fn metadata(&self) -> Metadata {
-        Metadata::named(format!("iggy-connectors-{}-config", self.connector_name))
+        let name = match self {
+            Self::Sink { connector_name, .. } => connector_name,
+            Self::Source { connector_name, .. } => connector_name,
+        };
+        Metadata::named(format!("iggy-connectors-{}-config", name))
     }
 
     fn data(&self) -> Result<figment::value::Map<Profile, Dict>, figment::Error> {
-        self.provider.deserialize().map_err(|_| {
-            figment::Error::from(format!(
-                "Cannot deserialize environment variables for connector config {}",
-                self.connector_name
-            ))
-        })
+        match self {
+            Self::Sink { provider, .. } => provider
+                .deserialize_with_runtime_prefix()
+                .map_err(|e| figment::Error::from(format!("Failed to deserialize env vars: {e}"))),
+            Self::Source { provider, .. } => provider
+                .deserialize_with_runtime_prefix()
+                .map_err(|e| figment::Error::from(format!("Failed to deserialize env vars: {e}"))),
+        }
     }
 }
