@@ -17,5 +17,14 @@
 
 //! Iggy metadata module
 
-mod impls;
+pub mod impls;
+pub mod permissioner;
 pub mod stm;
+
+mod stats;
+
+// Re-export IggyMetadata for use in other modules
+pub use impls::metadata::IggyMetadata;
+
+// Re-export MuxStateMachine for use in other modules
+pub use stm::mux::MuxStateMachine;
