@@ -41,7 +41,7 @@ use tokio::time::sleep;
 use tracing::{error, info, trace, warn};
 
 const ORDERING: std::sync::atomic::Ordering = std::sync::atomic::Ordering::SeqCst;
-type PollMessagesFuture = Pin<Box<dyn Future<Output = Result<PolledMessages, IggyError>>>>;
+type PollMessagesFuture = Pin<Box<dyn Future<Output = Result<PolledMessages, IggyError>> + Send>>;
 
 /// The auto-commit configuration for storing the offset on the server.
 #[derive(Debug, PartialEq, Copy, Clone)]
@@ -90,11 +90,18 @@ pub enum AutoCommitAfter {
     ConsumingEveryNthMessage(u32),
 }
 
-unsafe impl Send for IggyConsumer {}
+// SAFETY: IggyConsumer is Sync because:
+// 1. The only non-Sync field is `poll_future: Option<PollMessagesFuture>`
+// 2. `poll_future` is only accessed through `poll_next()` which requires `Pin<&mut Self>`
+//    (exclusive mutable access), so concurrent access to `poll_future` is impossible
+// 3. All other fields are inherently Sync (Arc<AtomicX>, Arc<DashMap>, etc.) or
+//    only accessed through `&mut self` methods
+// 4. All `&self` methods only access Sync-safe fields
 unsafe impl Sync for IggyConsumer {}
 
 pub struct IggyConsumer {
     initialized: bool,
+    shutdown: Arc<AtomicBool>,
     can_poll: Arc<AtomicBool>,
     client: IggyRwLock<ClientWrapper>,
     consumer_name: String,
@@ -153,6 +160,7 @@ impl IggyConsumer {
         let (store_offset_sender, _) = flume::unbounded();
         Self {
             initialized: false,
+            shutdown: Arc::new(AtomicBool::new(false)),
             is_consumer_group: consumer.kind == ConsumerKind::ConsumerGroup,
             joined_consumer_group: Arc::new(AtomicBool::new(false)),
             can_poll: Arc::new(AtomicBool::new(true)),
@@ -463,6 +471,7 @@ impl IggyConsumer {
         let topic_id = self.topic_id.clone();
         let last_consumed_offsets = self.last_consumed_offsets.clone();
         let last_stored_offsets = self.last_stored_offsets.clone();
+        let shutdown = self.shutdown.clone();
         tokio::spawn(async move {
             loop {
                 sleep(interval.get_duration()).await;
@@ -481,6 +490,10 @@ impl IggyConsumer {
                     )
                     .await;
                 }
+                if shutdown.load(ORDERING) {
+                    trace!("Shutdown signal received, stopping background offset storage");
+                    break;
+                }
             }
         });
     }
@@ -642,14 +655,22 @@ impl IggyConsumer {
         let last_stored_offset = self.last_stored_offsets.clone();
         let last_consumed_offset = self.last_consumed_offsets.clone();
         let allow_replay = self.allow_replay;
+        let is_consumer_group = self.is_consumer_group;
+        let joined_consumer_group = self.joined_consumer_group.clone();
 
         async move {
             if interval > 0 {
                 Self::wait_before_polling(interval, last_polled_at.load(ORDERING)).await;
             }
 
-            if !can_poll.load(ORDERING) {
-                trace!("Trying to poll messages in {retry_interval}...");
+            while !can_poll.load(ORDERING)
+                || (is_consumer_group && !joined_consumer_group.load(ORDERING))
+            {
+                trace!(
+                    "Cannot poll yet (can_poll={}, joined_cg={}), waiting {retry_interval}...",
+                    can_poll.load(ORDERING),
+                    joined_consumer_group.load(ORDERING)
+                );
                 sleep(retry_interval.get_duration()).await;
             }
 
@@ -759,10 +780,17 @@ impl IggyConsumer {
 
             let error = polled_messages.unwrap_err();
             error!("Failed to poll messages: {error}");
+
+            // Handle connection/auth errors - disable polling until event task re-enables
+            // it after reconnection and rejoin complete
             if matches!(
                 error,
                 IggyError::Disconnected | IggyError::Unauthenticated | IggyError::StaleClient
             ) {
+                can_poll.store(false, ORDERING);
+                if is_consumer_group {
+                    joined_consumer_group.store(false, ORDERING);
+                }
                 trace!("Retrying to poll messages in {retry_interval}...");
                 sleep(retry_interval.get_duration()).await;
             }
@@ -784,14 +812,6 @@ impl IggyConsumer {
             return;
         }
 
-        if now < last_sent_at {
-            warn!(
-                "Returned monotonic time went backwards, now < last_sent_at: ({now} < {last_sent_at})"
-            );
-            sleep(Duration::from_micros(interval)).await;
-            return;
-        }
-
         let elapsed = now - last_sent_at;
         if elapsed >= interval {
             trace!("No need to wait before polling messages. {now} - {last_sent_at} = {elapsed}");
@@ -923,7 +943,7 @@ impl Stream for IggyConsumer {
             }
 
             if self.buffered_messages.is_empty() {
-                if self.polling_strategy.kind == PollingKind::Offset {
+                if self.polling_strategy.kind != PollingKind::Next {
                     self.polling_strategy = PollingStrategy::offset(message.header.offset + 1);
                 }
 
@@ -990,7 +1010,7 @@ impl Stream for IggyConsumer {
                         let message = polled_messages.messages.remove(0);
                         self.buffered_messages.extend(polled_messages.messages);
 
-                        if self.polling_strategy.kind == PollingKind::Offset {
+                        if self.polling_strategy.kind != PollingKind::Next {
                             self.polling_strategy =
                                 PollingStrategy::offset(message.header.offset + 1);
                         }
@@ -1035,3 +1055,76 @@ impl Stream for IggyConsumer {
         Poll::Pending
     }
 }
+
+impl IggyConsumer {
+    pub async fn shutdown(&mut self) -> Result<(), IggyError> {
+        if self.shutdown.swap(true, ORDERING) {
+            return Ok(());
+        }
+
+        info!("Shutting down consumer: {}...", self.consumer_name);
+
+        for entry in self.last_consumed_offsets.iter() {
+            let partition_id = *entry.key();
+            let consumed_offset = entry.load(ORDERING);
+
+            let stored_offset = self
+                .last_stored_offsets
+                .get(&partition_id)
+                .map(|e| e.load(ORDERING))
+                .unwrap_or(0);
+
+            if consumed_offset > stored_offset {
+                trace!(
+                    "Flushing final offset: {consumed_offset} for partition: {partition_id}, stream: {}, topic: {}",
+                    self.stream_id, self.topic_id
+                );
+                let _ = Self::store_consumer_offset(
+                    &self.client,
+                    &self.consumer,
+                    &self.stream_id,
+                    &self.topic_id,
+                    partition_id,
+                    consumed_offset,
+                    &self.last_stored_offsets,
+                    self.allow_replay,
+                )
+                .await;
+            }
+        }
+
+        if self.is_consumer_group && self.joined_consumer_group.load(ORDERING) {
+            let group_id = self.consumer.id.clone();
+            trace!(
+                "Leaving consumer group: {group_id} for stream: {}, topic: {}",
+                self.stream_id, self.topic_id
+            );
+
+            let client = self.client.read().await;
+            if let Err(error) = client
+                .leave_consumer_group(&self.stream_id, &self.topic_id, &group_id)
+                .await
+            {
+                warn!(
+                    "Failed to leave consumer group: {group_id} for stream: {}, topic: {}. {error}",
+                    self.stream_id, self.topic_id
+                );
+            } else {
+                self.joined_consumer_group.store(false, ORDERING);
+            }
+        }
+
+        info!("Consumer: {} has been shut down.", self.consumer_name);
+        Ok(())
+    }
+}
+
+impl Drop for IggyConsumer {
+    fn drop(&mut self) {
+        self.shutdown.store(true, ORDERING);
+        trace!(
+            "Consumer {} has been dropped, shutdown signal sent",
+            self.consumer_name
+        );
+    }
+}
