@@ -19,7 +19,7 @@
 pub mod get_client_handler;
 pub mod get_clients_handler;
 pub mod get_me_handler;
-pub mod get_snapshot;
+pub mod get_snapshot_handler;
 pub mod get_stats_handler;
 pub mod ping_handler;
 
