@@ -23,9 +23,11 @@ use crate::{
     configs::{
         cache_indexes::CacheIndexesConfig,
         server::ServerConfig,
+        sharding::ShardInfo,
         system::{INDEX_EXTENSION, LOG_EXTENSION, SystemConfig},
     },
     io::fs_utils::{self, DirEntry},
+    metadata::{ConsumerGroupMeta, MetadataWriter, PartitionMeta, StreamMeta, TopicMeta, UserMeta},
     server_error::ServerError,
     shard::{
         system::info::SystemInfo,
@@ -34,229 +36,45 @@ use crate::{
             frame::ShardFrame,
         },
     },
-    slab::{
-        streams::Streams,
-        traits_ext::{
-            EntityComponentSystem, EntityComponentSystemMutCell, Insert, InsertCell, IntoComponents,
-        },
-        users::Users,
-    },
     state::system::{StreamState, TopicState, UserState},
     streaming::{
         partitions::{
-            consumer_offset::ConsumerOffset,
-            helpers::create_message_deduplicator,
-            journal::MemoryMessageJournal,
-            log::SegmentedLog,
-            partition,
-            storage::{load_consumer_group_offsets, load_consumer_offsets},
+            consumer_group_offsets::ConsumerGroupOffsets, consumer_offsets::ConsumerOffsets,
+            journal::MemoryMessageJournal, log::SegmentedLog,
         },
         persistence::persister::{FilePersister, FileWithSyncPersister, PersisterKind},
-        personal_access_tokens::personal_access_token::PersonalAccessToken,
-        polling_consumer::ConsumerGroupId,
         segments::{Segment, storage::Storage},
         stats::{PartitionStats, StreamStats, TopicStats},
         storage::SystemStorage,
-        streams::stream,
-        topics::{consumer_group, topic},
         users::user::User,
         utils::{crypto, file::overwrite},
     },
-    versioning::SemanticVersion,
 };
-use ahash::HashMap;
 use compio::{fs::create_dir_all, runtime::Runtime};
 use err_trail::ErrContext;
+use iggy_common::SemanticVersion;
 use iggy_common::{
-    IggyByteSize, IggyError,
+    IggyByteSize, IggyError, PersonalAccessToken,
     defaults::{
         DEFAULT_ROOT_USERNAME, MAX_PASSWORD_LENGTH, MAX_USERNAME_LENGTH, MIN_PASSWORD_LENGTH,
         MIN_USERNAME_LENGTH,
     },
 };
-use std::{collections::HashSet, env, path::Path, sync::Arc};
+use slab::Slab;
+use std::{env, path::Path, sync::Arc};
 use tracing::{info, warn};
 
-pub async fn load_streams(
-    state: impl IntoIterator<Item = StreamState>,
-    config: &SystemConfig,
-) -> Result<Streams, IggyError> {
-    let streams = Streams::default();
-    for StreamState {
-        name,
-        created_at,
-        id,
-        topics,
-    } in state
-    {
-        info!(
-            "Loading stream with ID: {}, name: {} from state...",
-            id, name
-        );
-        let stream_id = id;
-        let stats = Arc::new(StreamStats::default());
-        let stream = stream::Stream::new(name.clone(), stats.clone(), created_at);
-        let new_id = streams.insert(stream);
-        assert_eq!(
-            new_id, stream_id as usize,
-            "load_streams: id mismatch when inserting stream, mismatch for stream with ID: {}, name: {}",
-            stream_id, name
-        );
-        info!(
-            "Loaded stream with ID: {}, name: {} from state...",
-            id, name
-        );
-
-        let topics = topics.into_values();
-        for TopicState {
-            id,
-            name,
-            created_at,
-            compression_algorithm,
-            message_expiry,
-            max_topic_size,
-            replication_factor,
-            consumer_groups,
-            partitions,
-        } in topics
-        {
-            info!(
-                "Loading topic with ID: {}, name: {} from state...",
-                id, name
-            );
-            let topic_id = id;
-            let parent_stats = stats.clone();
-            let stats = Arc::new(TopicStats::new(parent_stats));
-            let topic_id = streams.with_components_by_id_mut(stream_id as usize, |(mut root, ..)| {
-                let topic = topic::Topic::new(
-                    name.clone(),
-                    stats.clone(),
-                    created_at,
-                    replication_factor.unwrap_or(1),
-                    message_expiry,
-                    compression_algorithm,
-                    max_topic_size,
-                );
-                let new_id = root.topics_mut().insert(topic);
-                assert_eq!(
-                    new_id, topic_id as usize,
-                    "load_streams: topic id mismatch when inserting topic, mismatch for topic with ID: {}, name: {}",
-                    topic_id, &name
-                );
-                new_id
-            });
-            info!("Loaded topic with ID: {}, name: {} from state...", id, name);
-
-            let parent_stats = stats.clone();
-            let cgs = consumer_groups.into_values();
-            let partitions = partitions.into_values();
-
-            // Load each partition asynchronously and insert immediately
-            for partition_state in partitions {
-                info!(
-                    "Loading partition with ID: {}, for topic with ID: {} from state...",
-                    partition_state.id, topic_id
-                );
-
-                let partition_id = partition_state.id;
-                let partition = load_partition(
-                    config,
-                    stream_id as usize,
-                    topic_id,
-                    partition_state,
-                    parent_stats.clone(),
-                )
-                .await?;
-                streams.with_components_by_id(stream_id as usize, |(root, ..)| {
-                    root.topics()
-                        .with_components_by_id_mut(topic_id, |(mut root, ..)| {
-                            let new_id = root.partitions_mut().insert(partition);
-                            assert_eq!(
-                                new_id, partition_id as usize,
-                                "load_streams: partition id mismatch when inserting partition, mismatch for partition with ID: {}, for topic with ID: {}, for stream with ID: {}",
-                                partition_id, topic_id, stream_id
-                            );
-                        });
-                });
-
-                info!(
-                    "Loaded partition with ID: {}, for topic with ID: {} from state...",
-                    partition_id, topic_id
-                );
-            }
-            let partition_ids = streams.with_components_by_id(stream_id as usize, |(root, ..)| {
-                root.topics().with_components_by_id(topic_id, |(root, ..)| {
-                    root.partitions().with_components(|components| {
-                        let (root, ..) = components.into_components();
-                        root.iter().map(|(_, root)| root.id()).collect::<Vec<_>>()
-                    })
-                })
-            });
-
-            for cg_state in cgs {
-                info!(
-                    "Loading consumer group with ID: {}, name: {} for topic with ID: {} from state...",
-                    cg_state.id, cg_state.name, topic_id
-                );
-                streams.with_components_by_id(stream_id as usize, |(root, ..)| {
-                    root.topics()
-                        .with_components_by_id_mut(topic_id, |(mut root, ..)| {
-                            let id = cg_state.id;
-                            let cg = consumer_group::ConsumerGroup::new(cg_state.name.clone(), Default::default(), partition_ids.clone());
-                            let new_id = root.consumer_groups_mut().insert(cg);
-                            assert_eq!(
-                                new_id, id as usize,
-                                "load_streams: consumer group id mismatch when inserting consumer group, mismatch for consumer group with ID: {}, name: {} for topic with ID: {}, for stream with ID: {}",
-                                id, cg_state.name, topic_id, stream_id
-                            );
-                        });
-                });
-                info!(
-                    "Loaded consumer group with ID: {}, name: {} for topic with ID: {} from state...",
-                    cg_state.id, cg_state.name, topic_id
-                );
-            }
-        }
-    }
-    Ok(streams)
-}
-
-pub fn load_users(state: impl IntoIterator<Item = UserState>) -> Users {
-    let users = Users::new();
-    for user_state in state {
-        let UserState {
-            id,
-            username,
-            password_hash,
-            status,
-            created_at,
-            permissions,
-            personal_access_tokens,
-        } = user_state;
-        let mut user = User::with_password(id, &username, password_hash, status, permissions);
-        user.created_at = created_at;
-        user.personal_access_tokens = personal_access_tokens
-            .into_values()
-            .map(|token| {
-                (
-                    Arc::new(token.token_hash.clone()),
-                    PersonalAccessToken::raw(id, &token.name, &token.token_hash, token.expiry_at),
-                )
-            })
-            .collect();
-        users.insert(user);
-    }
-    users
-}
-
 pub fn create_shard_connections(
-    shards_set: &HashSet<usize>,
+    shard_assignment: &[ShardInfo],
 ) -> (Vec<ShardConnector<ShardFrame>>, Vec<(u16, StopSender)>) {
-    let shards_count = shards_set.len();
-
     // Create connectors with sequential IDs (0, 1, 2, ...) regardless of CPU core numbers
-    let connectors: Vec<ShardConnector<ShardFrame>> = (0..shards_count)
-        .map(|idx| ShardConnector::new(idx as u16))
+    let connectors: Vec<ShardConnector<ShardFrame>> = shard_assignment
+        .iter()
+        .enumerate()
+        .map(|(idx, _assignment)| {
+            // let cpu_id = assignment.cpu_set.iter().next().unwrap_or(&idx);
+            ShardConnector::new(idx as u16)
+        })
         .collect();
 
     let shutdown_handles = connectors
@@ -347,7 +165,7 @@ pub fn create_root_user() -> User {
     User::root(&username, &password)
 }
 
-pub fn create_shard_executor(_cpu_set: HashSet<usize>) -> Runtime {
+pub fn create_shard_executor() -> Runtime {
     // TODO: The event interval tick, could be configured based on the fact
     // How many clients we expect to have connected.
     // This roughly estimates the number of tasks we will create.
@@ -367,7 +185,6 @@ pub fn create_shard_executor(_cpu_set: HashSet<usize>) -> Runtime {
     compio::runtime::RuntimeBuilder::new()
         .with_proactor(proactor.to_owned())
         .event_interval(128)
-        .thread_affinity(_cpu_set)
         .build()
         .unwrap()
 }
@@ -433,7 +250,6 @@ pub async fn load_segments(
 
         let messages_file_path = format!("{}/{}.{}", partition_path, log_file_name, LOG_EXTENSION);
         let index_file_path = format!("{}/{}.{}", partition_path, log_file_name, INDEX_EXTENSION);
-        let time_index_path = index_file_path.replace(INDEX_EXTENSION, "timeindex");
 
         async fn try_exists(path: &str) -> Result<bool, std::io::Error> {
             match compio::fs::metadata(path).await {
@@ -446,13 +262,12 @@ pub async fn load_segments(
         }
 
         let index_path_exists = try_exists(&index_file_path).await.unwrap();
-        let time_index_path_exists = try_exists(&time_index_path).await.unwrap();
         let index_cache_enabled = matches!(
             config.segment.cache_indexes,
             CacheIndexesConfig::All | CacheIndexesConfig::OpenSegment
         );
 
-        if index_cache_enabled && (!index_path_exists || time_index_path_exists) {
+        if index_cache_enabled && !index_path_exists {
             warn!(
                 "Index at path {} does not exist, rebuilding it based on {}...",
                 index_file_path, messages_file_path
@@ -476,10 +291,6 @@ pub async fn load_segments(
             );
         }
 
-        if time_index_path_exists {
-            compio::fs::remove_file(&time_index_path).await.unwrap();
-        }
-
         let messages_metadata = compio::fs::metadata(&messages_file_path)
             .await
             .map_err(|_| IggyError::CannotReadPartitions)?;
@@ -508,7 +319,7 @@ pub async fn load_segments(
             .unwrap()
             .load_all_indexes_from_disk()
             .await
-            .with_error(|error| format!("Failed to load indexes during startup for stream ID: {}, topic ID: {}, partition_id: {}, {error}", stream_id, topic_id, partition_id))
+            .error(|e: &IggyError| format!("Failed to load indexes during startup for stream ID: {}, topic ID: {}, partition_id: {}, {e}", stream_id, topic_id, partition_id))
             .map_err(|_| IggyError::CannotReadFile)?
         };
 
@@ -528,11 +339,7 @@ pub async fn load_segments(
             )
         };
 
-        let mut segment = Segment::new(
-            start_offset,
-            config.segment.size,
-            config.segment.message_expiry,
-        );
+        let mut segment = Segment::new(start_offset, config.segment.size);
 
         segment.start_timestamp = start_timestamp;
         segment.end_timestamp = end_timestamp;
@@ -564,7 +371,7 @@ pub async fn load_segments(
                     match messages_reader.load_messages_from_disk(batch_indexes).await {
                         Ok(messages_batch) => {
                             if let Err(e) = messages_batch.validate_checksums() {
-                                return Err(IggyError::CannotReadPartitions).with_error(|_| {
+                                return Err(IggyError::CannotReadPartitions).error(|_: &IggyError| {
                                     format!(
                                         "Failed to validate message checksum for segment at offset {} in stream ID: {}, topic ID: {}, partition ID: {}, error: {}",
                                         start_offset, stream_id, topic_id, partition_id, e
@@ -575,7 +382,7 @@ pub async fn load_segments(
                             current_relative_offset += batch_count;
                         }
                         Err(e) => {
-                            return Err(e).with_error(|_| {
+                            return Err(e).error(|_: &IggyError| {
                                 format!(
                                     "Failed to load messages from disk for checksum validation at offset {} in stream ID: {}, topic ID: {}, partition ID: {}",
                                     start_offset, stream_id, topic_id, partition_id
@@ -621,6 +428,11 @@ pub async fn load_segments(
         }
     }
 
+    // The last segment is the active one and must remain unsealed for writes
+    if log.has_segments() {
+        log.segments_mut().last_mut().unwrap().sealed = false;
+    }
+
     if matches!(
         config.segment.cache_indexes,
         CacheIndexesConfig::OpenSegment
@@ -645,86 +457,216 @@ pub async fn load_segments(
     Ok(log)
 }
 
-async fn load_partition(
-    config: &SystemConfig,
-    stream_id: usize,
-    topic_id: usize,
-    partition_state: crate::state::system::PartitionState,
-    parent_stats: Arc<TopicStats>,
-) -> Result<partition::Partition, IggyError> {
-    let stats = Arc::new(PartitionStats::new(parent_stats));
-    let partition_id = partition_state.id;
+/// Builds `InnerMetadata` from persisted user and stream state.
+pub fn build_inner_metadata(
+    users_state: impl IntoIterator<Item = UserState>,
+    streams_state: impl IntoIterator<Item = StreamState>,
+) -> crate::metadata::InnerMetadata {
+    use crate::metadata::InnerMetadata;
+    use std::sync::atomic::AtomicUsize;
 
-    let partition_path = config.get_partition_path(stream_id, topic_id, partition_id as usize);
-    let log_files = collect_log_files(&partition_path).await?;
-    let should_increment_offset = !log_files.is_empty()
-        && log_files
-            .first()
-            .map(|entry| {
-                let log_file_name = entry
-                    .path
-                    .file_stem()
-                    .unwrap()
-                    .to_string_lossy()
-                    .to_string();
+    let mut user_entries = Vec::new();
+    let mut user_index = ahash::AHashMap::default();
+    let mut personal_access_tokens: ahash::AHashMap<
+        u32,
+        ahash::AHashMap<Arc<str>, PersonalAccessToken>,
+    > = ahash::AHashMap::default();
+    let mut users_count = 0;
+    let mut pats_count = 0;
 
-                let start_offset = log_file_name.parse::<u64>().unwrap();
+    for UserState {
+        id,
+        username,
+        password_hash,
+        status,
+        created_at,
+        permissions,
+        personal_access_tokens: user_pats,
+    } in users_state
+    {
+        let username_arc: Arc<str> = Arc::from(username.as_str());
+        let user_meta = UserMeta {
+            id,
+            username: username_arc.clone(),
+            password_hash: Arc::from(password_hash.as_str()),
+            status,
+            permissions: permissions.map(Arc::new),
+            created_at,
+        };
+        user_entries.push((id as usize, user_meta));
+        user_index.insert(username_arc, id);
 
-                let messages_file_path = config.get_messages_file_path(
-                    stream_id,
-                    topic_id,
-                    partition_id as usize,
-                    start_offset,
+        if !user_pats.is_empty() {
+            let user_pat_map: ahash::AHashMap<Arc<str>, PersonalAccessToken> = user_pats
+                .into_values()
+                .map(|token| {
+                    let pat = PersonalAccessToken::raw(
+                        id,
+                        &token.name,
+                        &token.token_hash,
+                        token.expiry_at,
+                    );
+                    (Arc::from(token.token_hash.as_str()), pat)
+                })
+                .collect();
+            pats_count += user_pat_map.len();
+            personal_access_tokens.insert(id, user_pat_map);
+        }
+
+        users_count += 1;
+    }
+    info!(
+        "Building metadata: {} users, {} personal access tokens",
+        users_count, pats_count
+    );
+
+    let mut stream_entries = Vec::new();
+    let mut stream_index = ahash::AHashMap::default();
+    let mut streams_count = 0;
+    let mut topics_count = 0;
+    let mut partitions_count = 0;
+    let mut consumer_groups_count = 0;
+
+    for StreamState {
+        name,
+        created_at,
+        id,
+        topics,
+    } in streams_state
+    {
+        info!(
+            "Building stream with ID: {}, name: {} metadata...",
+            id, name
+        );
+        let stream_id = id as usize;
+        let stream_name: Arc<str> = Arc::from(name.as_str());
+
+        let stream_stats = Arc::new(StreamStats::default());
+
+        let mut topic_entries = Vec::new();
+        let mut topic_index = ahash::AHashMap::default();
+
+        for TopicState {
+            id,
+            name,
+            created_at,
+            compression_algorithm,
+            message_expiry,
+            max_topic_size,
+            replication_factor,
+            consumer_groups,
+            partitions,
+        } in topics.into_values()
+        {
+            info!("Building topic with ID: {}, name: {} metadata...", id, name);
+            let topic_id = id as usize;
+            let topic_name: Arc<str> = Arc::from(name.as_str());
+
+            let topic_stats = Arc::new(TopicStats::new(stream_stats.clone()));
+
+            let mut partition_entries = Vec::new();
+            let mut partition_ids = Vec::new();
+
+            for partition_state in partitions.into_values() {
+                let partition_id = partition_state.id as usize;
+                partition_ids.push(partition_id);
+
+                let partition_stats = Arc::new(PartitionStats::new(topic_stats.clone()));
+                let partition_meta = PartitionMeta {
+                    id: partition_id,
+                    created_at: partition_state.created_at,
+                    revision_id: 0,
+                    stats: partition_stats,
+                    consumer_offsets: Arc::new(ConsumerOffsets::with_capacity(0)),
+                    consumer_group_offsets: Arc::new(ConsumerGroupOffsets::with_capacity(0)),
+                    last_polled_offsets: Arc::new(papaya::HashMap::new()),
+                };
+                partition_entries.push((partition_id, partition_meta));
+                partitions_count += 1;
+            }
+
+            partition_ids.sort_unstable();
+
+            let mut cg_entries = Vec::new();
+            let mut cg_index = ahash::AHashMap::default();
+
+            for cg_state in consumer_groups.into_values() {
+                info!(
+                    "Building consumer group with ID: {}, name: {} for topic with ID: {} metadata...",
+                    cg_state.id, cg_state.name, topic_id
                 );
-                let metadata = std::fs::metadata(&messages_file_path)
-                    .expect("failed to get metadata for first segment in log");
-                metadata.len() > 0
-            })
-            .unwrap_or_else(|| false);
+                let group_id = cg_state.id as usize;
+                let group_name: Arc<str> = Arc::from(cg_state.name.as_str());
+                let cg_meta = ConsumerGroupMeta {
+                    id: group_id,
+                    name: group_name.clone(),
+                    partitions: partition_ids.clone(),
+                    members: Slab::new(),
+                };
+                cg_entries.push((group_id, cg_meta));
+                cg_index.insert(group_name, group_id);
+                consumer_groups_count += 1;
+            }
+
+            let topic_meta = TopicMeta {
+                id: topic_id,
+                name: topic_name.clone(),
+                created_at,
+                message_expiry,
+                compression_algorithm,
+                max_topic_size,
+                replication_factor: replication_factor.unwrap_or(1),
+                stats: topic_stats,
+                partitions: partition_entries.into_iter().map(|(_, p)| p).collect(),
+                consumer_groups: cg_entries.into_iter().collect(),
+                consumer_group_index: cg_index,
+                round_robin_counter: Arc::new(AtomicUsize::new(0)),
+            };
+            topic_entries.push((topic_id, topic_meta));
+            topic_index.insert(topic_name, topic_id);
+            topics_count += 1;
+        }
+
+        let stream_meta = StreamMeta {
+            id: stream_id,
+            name: stream_name.clone(),
+            created_at,
+            stats: stream_stats,
+            topics: topic_entries.into_iter().collect(),
+            topic_index,
+        };
+        stream_entries.push((stream_id, stream_meta));
+        stream_index.insert(stream_name, stream_id);
+        streams_count += 1;
+    }
 
     info!(
-        "Loading partition with ID: {} for stream with ID: {} and topic with ID: {}, for path: {} from disk...",
-        partition_id, stream_id, topic_id, partition_path
+        "Built metadata: {} streams, {} topics, {} partitions, {} consumer groups",
+        streams_count, topics_count, partitions_count, consumer_groups_count
     );
 
-    // Load consumer offsets
-    let message_deduplicator = create_message_deduplicator(config);
-    let consumer_offset_path =
-        config.get_consumer_offsets_path(stream_id, topic_id, partition_id as usize);
-    let consumer_group_offsets_path =
-        config.get_consumer_group_offsets_path(stream_id, topic_id, partition_id as usize);
+    InnerMetadata {
+        streams: stream_entries.into_iter().collect(),
+        users: user_entries.into_iter().collect(),
+        stream_index,
+        user_index,
+        personal_access_tokens,
+        users_global_permissions: Default::default(),
+        users_stream_permissions: Default::default(),
+        users_can_poll_all_streams: Default::default(),
+        users_can_send_all_streams: Default::default(),
+        users_can_poll_stream: Default::default(),
+        users_can_send_stream: Default::default(),
+    }
+}
 
-    let consumer_offset = Arc::new(
-        load_consumer_offsets(&consumer_offset_path)?
-            .into_iter()
-            .map(|offset| (offset.consumer_id as usize, offset))
-            .collect::<HashMap<usize, ConsumerOffset>>()
-            .into(),
-    );
-
-    let consumer_group_offset = Arc::new(
-        load_consumer_group_offsets(&consumer_group_offsets_path)?
-            .into_iter()
-            .collect::<HashMap<ConsumerGroupId, ConsumerOffset>>()
-            .into(),
-    );
-
-    let log = Default::default();
-    let partition = partition::Partition::new(
-        partition_state.created_at,
-        should_increment_offset,
-        stats,
-        message_deduplicator,
-        Arc::new(Default::default()),
-        consumer_offset,
-        consumer_group_offset,
-        log,
-    );
-
-    info!(
-        "Loaded partition with ID: {} for stream with ID: {} and topic with ID: {}",
-        partition_id, stream_id, topic_id
-    );
-
-    Ok(partition)
+/// Loads all metadata from persisted state into metadata writer.
+/// Single atomic initialization via `Initialize` operation.
+pub fn load_metadata(
+    users_state: impl IntoIterator<Item = UserState>,
+    streams_state: impl IntoIterator<Item = StreamState>,
+    writer: &mut MetadataWriter,
+) {
+    let inner = build_inner_metadata(users_state, streams_state);
+    writer.initialize(inner);
 }
