@@ -25,9 +25,10 @@ use crate::configs::http::{
 };
 use crate::configs::quic::{QuicCertificateConfig, QuicConfig, QuicSocketConfig};
 use crate::configs::server::{
-    DataMaintenanceConfig, HeartbeatConfig, MemoryPoolConfig, MessageSaverConfig,
-    MessagesMaintenanceConfig, PersonalAccessTokenCleanerConfig, PersonalAccessTokenConfig,
-    ServerConfig, TelemetryConfig, TelemetryLogsConfig, TelemetryTracesConfig,
+    ConsumerGroupConfig, DataMaintenanceConfig, HeartbeatConfig, MemoryPoolConfig,
+    MessageSaverConfig, MessagesMaintenanceConfig, PersonalAccessTokenCleanerConfig,
+    PersonalAccessTokenConfig, ServerConfig, TelemetryConfig, TelemetryLogsConfig,
+    TelemetryTracesConfig,
 };
 use crate::configs::system::{
     BackupConfig, CompatibilityConfig, CompressionConfig, EncryptionConfig, LoggingConfig,
@@ -42,13 +43,14 @@ use std::sync::Arc;
 use std::time::Duration;
 
 static_toml::static_toml! {
-    // static_toml crate always starts from CARGO_MANIFEST_DIR (in this case iggy-server root directory)
-    pub static SERVER_CONFIG = include_toml!("../configs/server.toml");
+    // static_toml crate always starts from CARGO_MANIFEST_DIR (core/server)
+    pub static SERVER_CONFIG = include_toml!("config.toml");
 }
 
 impl Default for ServerConfig {
     fn default() -> ServerConfig {
         ServerConfig {
+            consumer_group: ConsumerGroupConfig::default(),
             data_maintenance: DataMaintenanceConfig::default(),
             heartbeat: HeartbeatConfig::default(),
             message_saver: MessageSaverConfig::default(),
@@ -129,6 +131,7 @@ impl Default for TcpConfig {
             ipv6: SERVER_CONFIG.tcp.ipv_6,
             tls: TcpTlsConfig::default(),
             socket: TcpSocketConfig::default(),
+            socket_migration: SERVER_CONFIG.tcp.socket_migration,
         }
     }
 }
@@ -190,6 +193,7 @@ impl Default for HttpConfig {
             enabled: SERVER_CONFIG.http.enabled,
             address: SERVER_CONFIG.http.address.parse().unwrap(),
             max_request_size: SERVER_CONFIG.http.max_request_size.parse().unwrap(),
+            web_ui: SERVER_CONFIG.http.web_ui,
             cors: HttpCorsConfig::default(),
             jwt: HttpJwtConfig::default(),
             metrics: HttpMetricsConfig::default(),
@@ -372,6 +376,23 @@ impl Default for HeartbeatConfig {
     }
 }
 
+impl Default for ConsumerGroupConfig {
+    fn default() -> ConsumerGroupConfig {
+        ConsumerGroupConfig {
+            rebalancing_timeout: SERVER_CONFIG
+                .consumer_group
+                .rebalancing_timeout
+                .parse()
+                .unwrap(),
+            rebalancing_check_interval: SERVER_CONFIG
+                .consumer_group
+                .rebalancing_check_interval
+                .parse()
+                .unwrap(),
+        }
+    }
+}
+
 impl Default for RuntimeConfig {
     fn default() -> RuntimeConfig {
         RuntimeConfig {
@@ -399,7 +420,15 @@ impl Default for LoggingConfig {
         LoggingConfig {
             path: SERVER_CONFIG.system.logging.path.parse().unwrap(),
             level: SERVER_CONFIG.system.logging.level.parse().unwrap(),
-            max_size: SERVER_CONFIG.system.logging.max_size.parse().unwrap(),
+            file_enabled: SERVER_CONFIG.system.logging.file_enabled,
+            max_file_size: SERVER_CONFIG.system.logging.max_file_size.parse().unwrap(),
+            max_total_size: SERVER_CONFIG.system.logging.max_total_size.parse().unwrap(),
+            rotation_check_interval: SERVER_CONFIG
+                .system
+                .logging
+                .rotation_check_interval
+                .parse()
+                .unwrap(),
             retention: SERVER_CONFIG.system.logging.retention.parse().unwrap(),
             sysinfo_print_interval: SERVER_CONFIG
                 .system
@@ -433,7 +462,7 @@ impl Default for TopicConfig {
         TopicConfig {
             path: SERVER_CONFIG.system.topic.path.parse().unwrap(),
             max_size: SERVER_CONFIG.system.topic.max_size.parse().unwrap(),
-            delete_oldest_segments: SERVER_CONFIG.system.topic.delete_oldest_segments,
+            message_expiry: SERVER_CONFIG.system.topic.message_expiry.parse().unwrap(),
         }
     }
 }
@@ -461,7 +490,6 @@ impl Default for SegmentConfig {
         SegmentConfig {
             size: SERVER_CONFIG.system.segment.size.parse().unwrap(),
             cache_indexes: SERVER_CONFIG.system.segment.cache_indexes.parse().unwrap(),
-            message_expiry: SERVER_CONFIG.system.segment.message_expiry.parse().unwrap(),
             archive_expired: SERVER_CONFIG.system.segment.archive_expired,
         }
     }
