@@ -22,65 +22,100 @@ use super::server::{
     DataMaintenanceConfig, MessageSaverConfig, MessagesMaintenanceConfig, TelemetryConfig,
 };
 use super::sharding::{CpuAllocation, ShardingConfig};
-use super::system::{CompressionConfig, PartitionConfig};
+use super::system::{CompressionConfig, LoggingConfig, PartitionConfig};
 use crate::configs::COMPONENT;
 use crate::configs::server::{MemoryPoolConfig, PersonalAccessTokenConfig, ServerConfig};
+use crate::configs::sharding::NumaTopology;
 use crate::configs::system::SegmentConfig;
 use crate::streaming::segments::*;
+use configs::ConfigurationError;
 use err_trail::ErrContext;
+use iggy_common::CompressionAlgorithm;
 use iggy_common::IggyExpiry;
 use iggy_common::MaxTopicSize;
 use iggy_common::Validatable;
-use iggy_common::{CompressionAlgorithm, ConfigurationError};
 use std::thread::available_parallelism;
-use tracing::error;
+use tracing::warn;
 
 impl Validatable<ConfigurationError> for ServerConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
-        self.system.memory_pool.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate memory pool config")
+        self.system
+            .memory_pool
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate memory pool config")
+            })?;
+        self.data_maintenance
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate data maintenance config")
+            })?;
+        self.personal_access_token
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!(
+                    "{COMPONENT} (error: {e}) - failed to validate personal access token config"
+                )
+            })?;
+        self.system
+            .segment
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate segment config")
+            })?;
+        self.system
+            .compression
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate compression config")
+            })?;
+        self.telemetry.validate().error(|e: &ConfigurationError| {
+            format!("{COMPONENT} (error: {e}) - failed to validate telemetry config")
         })?;
-        self.data_maintenance.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate data maintenance config")
-        })?;
-        self.personal_access_token.validate().with_error(|error| {
-            format!(
-                "{COMPONENT} (error: {error}) - failed to validate personal access token config"
-            )
-        })?;
-        self.system.segment.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate segment config")
-        })?;
-        self.system.compression.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate compression config")
-        })?;
-        self.telemetry.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate telemetry config")
-        })?;
-        self.system.sharding.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate sharding config")
-        })?;
-        self.cluster.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate cluster config")
+        self.system
+            .sharding
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate sharding config")
+            })?;
+        self.cluster.validate().error(|e: &ConfigurationError| {
+            format!("{COMPONENT} (error: {e}) - failed to validate cluster config")
         })?;
 
+        self.system
+            .logging
+            .validate()
+            .error(|e: &ConfigurationError| {
+                format!("{COMPONENT} (error: {e}) - failed to validate logging config")
+            })?;
+
         let topic_size = match self.system.topic.max_size {
             MaxTopicSize::Custom(size) => Ok(size.as_bytes_u64()),
             MaxTopicSize::Unlimited => Ok(u64::MAX),
-            MaxTopicSize::ServerDefault => Err(ConfigurationError::InvalidConfigurationValue),
+            MaxTopicSize::ServerDefault => {
+                eprintln!("system.topic.max_size cannot be ServerDefault in server config");
+                Err(ConfigurationError::InvalidConfigurationValue)
+            }
         }?;
 
-        if let IggyExpiry::ServerDefault = self.system.segment.message_expiry {
+        if let IggyExpiry::ServerDefault = self.system.topic.message_expiry {
+            eprintln!("system.topic.message_expiry cannot be ServerDefault in server config");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
         if self.http.enabled
             && let IggyExpiry::ServerDefault = self.http.jwt.access_token_expiry
         {
+            eprintln!("http.jwt.access_token_expiry cannot be ServerDefault when HTTP is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
         if topic_size < self.system.segment.size.as_bytes_u64() {
+            eprintln!(
+                "system.topic.max_size ({} B) must be >= system.segment.size ({} B)",
+                topic_size,
+                self.system.segment.size.as_bytes_u64()
+            );
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -93,7 +128,7 @@ impl Validatable<ConfigurationError> for CompressionConfig {
         let compression_alg = &self.default_algorithm;
         if *compression_alg != CompressionAlgorithm::None {
             // TODO(numinex): Change this message once server side compression is fully developed.
-            println!(
+            warn!(
                 "Server started with server-side compression enabled, using algorithm: {compression_alg}, this feature is not implemented yet!"
             );
         }
@@ -109,14 +144,17 @@ impl Validatable<ConfigurationError> for TelemetryConfig {
         }
 
         if self.service_name.trim().is_empty() {
+            eprintln!("telemetry.service_name cannot be empty when telemetry is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
         if self.logs.endpoint.is_empty() {
+            eprintln!("telemetry.logs.endpoint cannot be empty when telemetry is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
         if self.traces.endpoint.is_empty() {
+            eprintln!("telemetry.traces.endpoint cannot be empty when telemetry is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -126,39 +164,8 @@ impl Validatable<ConfigurationError> for TelemetryConfig {
 
 impl Validatable<ConfigurationError> for PartitionConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
-        if self.messages_required_to_save < 32 {
-            eprintln!(
-                "Configured system.partition.messages_required_to_save {} is less than minimum {}",
-                self.messages_required_to_save, 32
-            );
-            return Err(ConfigurationError::InvalidConfigurationValue);
-        }
-
-        if !self.messages_required_to_save.is_multiple_of(32) {
-            eprintln!(
-                "Configured system.partition.messages_required_to_save {} is not a multiple of 32",
-                self.messages_required_to_save
-            );
-            return Err(ConfigurationError::InvalidConfigurationValue);
-        }
-
-        if self.size_of_messages_required_to_save < 512 {
-            eprintln!(
-                "Configured system.partition.size_of_messages_required_to_save {} is less than minimum {}",
-                self.size_of_messages_required_to_save, 512
-            );
-            return Err(ConfigurationError::InvalidConfigurationValue);
-        }
-
-        if !self
-            .size_of_messages_required_to_save
-            .as_bytes_u64()
-            .is_multiple_of(512)
-        {
-            eprintln!(
-                "Configured system.partition.size_of_messages_required_to_save {} is not a multiple of 512 B",
-                self.size_of_messages_required_to_save
-            );
+        if self.messages_required_to_save == 0 {
+            eprintln!("Configured system.partition.messages_required_to_save cannot be 0");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -192,6 +199,7 @@ impl Validatable<ConfigurationError> for SegmentConfig {
 impl Validatable<ConfigurationError> for MessageSaverConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
         if self.enabled && self.interval.is_zero() {
+            eprintln!("message_saver.interval cannot be zero when message_saver is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -201,8 +209,8 @@ impl Validatable<ConfigurationError> for MessageSaverConfig {
 
 impl Validatable<ConfigurationError> for DataMaintenanceConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
-        self.messages.validate().with_error(|error| {
-            format!("{COMPONENT} (error: {error}) - failed to validate messages maintenance config")
+        self.messages.validate().error(|e: &ConfigurationError| {
+            format!("{COMPONENT} (error: {e}) - failed to validate messages maintenance config")
         })?;
         Ok(())
     }
@@ -211,6 +219,7 @@ impl Validatable<ConfigurationError> for DataMaintenanceConfig {
 impl Validatable<ConfigurationError> for MessagesMaintenanceConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
         if self.cleaner_enabled && self.interval.is_zero() {
+            eprintln!("data_maintenance.messages.interval cannot be zero when cleaner is enabled");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -221,10 +230,52 @@ impl Validatable<ConfigurationError> for MessagesMaintenanceConfig {
 impl Validatable<ConfigurationError> for PersonalAccessTokenConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
         if self.max_tokens_per_user == 0 {
+            eprintln!("personal_access_token.max_tokens_per_user cannot be 0");
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
         if self.cleaner.enabled && self.cleaner.interval.is_zero() {
+            eprintln!(
+                "personal_access_token.cleaner.interval cannot be zero when cleaner is enabled"
+            );
+            return Err(ConfigurationError::InvalidConfigurationValue);
+        }
+
+        Ok(())
+    }
+}
+
+impl Validatable<ConfigurationError> for LoggingConfig {
+    fn validate(&self) -> Result<(), ConfigurationError> {
+        if self.level.is_empty() {
+            eprintln!("system.logging.level is supposed be configured");
+            return Err(ConfigurationError::InvalidConfigurationValue);
+        }
+
+        if self.retention.as_secs() < 1 {
+            eprintln!(
+                "Configured system.logging.retention {} is less than minimum 1 second",
+                self.retention
+            );
+            return Err(ConfigurationError::InvalidConfigurationValue);
+        }
+
+        if self.rotation_check_interval.as_secs() < 1 {
+            eprintln!(
+                "Configured system.logging.rotation_check_interval {} is less than minimum 1 second",
+                self.rotation_check_interval
+            );
+            return Err(ConfigurationError::InvalidConfigurationValue);
+        }
+
+        let max_total_size_unlimited = self.max_total_size.as_bytes_u64() == 0;
+        if !max_total_size_unlimited
+            && self.max_file_size.as_bytes_u64() > self.max_total_size.as_bytes_u64()
+        {
+            eprintln!(
+                "Configured system.logging.max_total_size {} is less than system.logging.max_file_size {}",
+                self.max_total_size, self.max_file_size
+            );
             return Err(ConfigurationError::InvalidConfigurationValue);
         }
 
@@ -235,7 +286,7 @@ impl Validatable<ConfigurationError> for PersonalAccessTokenConfig {
 impl Validatable<ConfigurationError> for MemoryPoolConfig {
     fn validate(&self) -> Result<(), ConfigurationError> {
         if self.enabled && self.size == 0 {
-            error!(
+            eprintln!(
                 "Configured system.memory_pool.enabled is true and system.memory_pool.size is 0"
             );
             return Err(ConfigurationError::InvalidConfigurationValue);
@@ -246,7 +297,7 @@ impl Validatable<ConfigurationError> for MemoryPoolConfig {
         const DEFAULT_PAGE_SIZE: u64 = 4096;
 
         if self.enabled && self.size < MIN_POOL_SIZE {
-            error!(
+            eprintln!(
                 "Configured system.memory_pool.size {} B ({} MiB) is less than minimum {} B, ({} MiB)",
                 self.size.as_bytes_u64(),
                 self.size.as_bytes_u64() / (1024 * 1024),
@@ -257,7 +308,7 @@ impl Validatable<ConfigurationError> for MemoryPoolConfig {
         }
 
         if self.enabled && !self.size.as_bytes_u64().is_multiple_of(DEFAULT_PAGE_SIZE) {
-            error!(
+            eprintln!(
                 "Configured system.memory_pool.size {} B is not a multiple of default page size {} B",
                 self.size.as_bytes_u64(),
                 DEFAULT_PAGE_SIZE
@@ -266,7 +317,7 @@ impl Validatable<ConfigurationError> for MemoryPoolConfig {
         }
 
         if self.enabled && self.bucket_capacity < MIN_BUCKET_CAPACITY {
-            error!(
+            eprintln!(
                 "Configured system.memory_pool.buffers {} is less than minimum {}",
                 self.bucket_capacity, MIN_BUCKET_CAPACITY
             );
@@ -274,7 +325,7 @@ impl Validatable<ConfigurationError> for MemoryPoolConfig {
         }
 
         if self.enabled && !self.bucket_capacity.is_power_of_two() {
-            error!(
+            eprintln!(
                 "Configured system.memory_pool.buffers {} is not a power of 2",
                 self.bucket_capacity
             );
@@ -321,6 +372,18 @@ impl Validatable<ConfigurationError> for ShardingConfig {
                 }
                 Ok(())
             }
+            CpuAllocation::NumaAware(numa_config) => match NumaTopology::detect() {
+                // TODO: dry the validation, already validate it from the shard allocation
+                Ok(topology) => numa_config.validate(&topology).map_err(|e| {
+                    eprintln!("Invalid NUMA configuration: {}", e);
+                    ConfigurationError::InvalidConfigurationValue
+                }),
+                Err(e) => {
+                    eprintln!("Failed to detect NUMA topology: {}", e);
+                    eprintln!("NUMA allocation requested but system doesn't support it");
+                    Err(ConfigurationError::InvalidConfigurationValue)
+                }
+            },
         }
     }
 }
