@@ -21,6 +21,7 @@ mod jwt_token_cleaner;
 mod message_cleaner;
 mod message_saver;
 mod personal_access_token_cleaner;
+mod revocation_timeout;
 mod sysinfo_printer;
 
 pub use heartbeat_verifier::spawn_heartbeat_verifier;
@@ -28,4 +29,5 @@ pub use jwt_token_cleaner::spawn_jwt_token_cleaner;
 pub use message_cleaner::spawn_message_cleaner;
 pub use message_saver::spawn_message_saver;
 pub use personal_access_token_cleaner::spawn_personal_access_token_cleaner;
+pub use revocation_timeout::spawn_revocation_timeout_checker;
 pub use sysinfo_printer::spawn_sysinfo_printer;
