@@ -21,7 +21,6 @@ pub mod deduplication;
 pub mod diagnostics;
 pub mod partitions;
 pub mod persistence;
-pub mod personal_access_tokens;
 pub mod polling_consumer;
 pub mod segments;
 pub mod session;
@@ -29,6 +28,5 @@ pub mod stats;
 pub mod storage;
 pub mod streams;
 pub mod topics;
-pub mod traits;
 pub mod users;
 pub mod utils;
