@@ -18,8 +18,6 @@
 
 mod index_reader;
 mod index_writer;
-mod indexes_mut;
 
-pub use index_reader::IndexReader;
+pub use iggy_common::IggyIndexesMut;
 pub use index_writer::IndexWriter;
-pub use indexes_mut::IggyIndexesMut;
