@@ -16,6 +16,4 @@
  * under the License.
  */
 
-pub mod permissioner;
-pub mod permissioner_rules;
 pub mod user;
