@@ -24,12 +24,13 @@ using Apache.Iggy.Enums;
 using Apache.Iggy.Extensions;
 using Apache.Iggy.Headers;
 using Apache.Iggy.Messages;
+using Apache.Iggy.Utils;
 
 namespace Apache.Iggy.Mappers;
 
 internal static class BinaryMapper
 {
-    private const int PropertiesSize = 56;
+    private const int PropertiesSize = 64;
 
     internal static RawPersonalAccessToken MapRawPersonalAccessToken(ReadOnlySpan<byte> payload)
     {
@@ -359,17 +360,18 @@ internal static class BinaryMapper
             var originTimestamp = BinaryPrimitives.ReadUInt64LittleEndian(payload[(position + 40)..(position + 48)]);
             var headersLength = BinaryPrimitives.ReadInt32LittleEndian(payload[(position + 48)..(position + 52)]);
             var payloadLength = BinaryPrimitives.ReadInt32LittleEndian(payload[(position + 52)..(position + 56)]);
+            var reserved = BinaryPrimitives.ReadUInt64LittleEndian(payload[(position + 56)..(position + 64)]);
 
             Dictionary<HeaderKey, HeaderValue>? headers = headersLength switch
             {
                 0 => null,
                 > 0 => MapHeaders(
-                    payload[(position + 56 + payloadLength)..(position + 56 + payloadLength + headersLength)]),
+                    payload[(position + 64 + payloadLength)..(position + 64 + payloadLength + headersLength)]),
                 < 0 => throw new ArgumentOutOfRangeException()
             };
 
-            var payloadRangeStart = position + 56;
-            var payloadRangeEnd = position + 56 + payloadLength;
+            var payloadRangeStart = position + 64;
+            var payloadRangeEnd = position + 64 + payloadLength;
             if (payloadRangeStart > length || payloadRangeEnd > length)
             {
                 break;
@@ -393,7 +395,8 @@ internal static class BinaryMapper
                         OriginTimestamp = originTimestamp,
                         PayloadLength = payloadLength,
                         Timestamp = DateTimeOffsetUtils.FromUnixTimeMicroSeconds(timestamp),
-                        UserHeadersLength = headersLength
+                        UserHeadersLength = headersLength,
+                        Reserved = reserved
                     },
                     UserHeaders = headers,
                     Payload = decryptor is not null
@@ -406,7 +409,7 @@ internal static class BinaryMapper
                 ArrayPool<byte>.Shared.Return(messagePayload);
             }
 
-            position += 56 + payloadLength + headersLength;
+            position += 64 + payloadLength + headersLength;
             if (position + PropertiesSize >= length)
             {
                 break;
@@ -428,17 +431,22 @@ internal static class BinaryMapper
 
         while (position < payload.Length)
         {
+            var keyKind = MapHeaderKind(payload, position);
+            position++;
+
             var keyLength = BinaryPrimitives.ReadInt32LittleEndian(payload[position..(position + 4)]);
             if (keyLength is 0 or > 255)
             {
                 throw new ArgumentException("Key has incorrect size, must be between 1 and 255", nameof(keyLength));
             }
 
-            var key = Encoding.UTF8.GetString(payload[(position + 4)..(position + 4 + keyLength)]);
-            position += 4 + keyLength;
+            position += 4;
+            var keyValue = payload[position..(position + keyLength)].ToArray();
+            position += keyLength;
 
-            var headerKind = MapHeaderKind(payload, position);
+            var valueKind = MapHeaderKind(payload, position);
             position++;
+
             var valueLength = BinaryPrimitives.ReadInt32LittleEndian(payload[position..(position + 4)]);
             if (valueLength is 0 or > 255)
             {
@@ -448,11 +456,9 @@ internal static class BinaryMapper
             position += 4;
             ReadOnlySpan<byte> value = payload[position..(position + valueLength)];
             position += valueLength;
-            headers.Add(HeaderKey.New(key), new HeaderValue
-            {
-                Kind = headerKind,
-                Value = value.ToArray()
-            });
+
+            headers[new HeaderKey { Kind = keyKind, Value = keyValue }] =
+                new HeaderValue { Kind = valueKind, Value = value.ToArray() };
         }
 
         return headers;
@@ -465,9 +471,13 @@ internal static class BinaryMapper
             1 => HeaderKind.Raw,
             2 => HeaderKind.String,
             3 => HeaderKind.Bool,
+            4 => HeaderKind.Int8,
+            5 => HeaderKind.Int16,
             6 => HeaderKind.Int32,
             7 => HeaderKind.Int64,
             8 => HeaderKind.Int128,
+            9 => HeaderKind.Uint8,
+            10 => HeaderKind.Uint16,
             11 => HeaderKind.Uint32,
             12 => HeaderKind.Uint64,
             13 => HeaderKind.Uint128,
@@ -613,7 +623,7 @@ internal static class BinaryMapper
                 MessagesCount = messagesCount,
                 Size = sizeBytes,
                 CreatedAt = DateTimeOffsetUtils.FromUnixTimeMicroSeconds(createdAt).LocalDateTime,
-                MessageExpiry = messageExpiry,
+                MessageExpiry = DurationHelpers.FromDuration(messageExpiry),
                 ReplicationFactor = replicationFactor,
                 MaxTopicSize = maxTopicSize
             }, readBytes);
