@@ -26,6 +26,6 @@ class TopicsHttpClientTest extends TopicsClientBaseTest {
 
     @Override
     protected IggyBaseClient getClient() {
-        return HttpClientFactory.create(iggyServer);
+        return HttpClientFactory.create(serverHost(), serverHttpPort());
     }
 }
