@@ -19,21 +19,15 @@
 
 package org.apache.iggy.client.blocking.tcp;
 
-import org.testcontainers.containers.GenericContainer;
-
-import static org.apache.iggy.client.blocking.IntegrationTest.TCP_PORT;
+import org.apache.iggy.Iggy;
 
 final class TcpClientFactory {
 
     private TcpClientFactory() {}
 
-    static IggyTcpClient create(GenericContainer<?> iggyServer) {
-        if (iggyServer == null) {
-            // Server is running externally
-            return new IggyTcpClient("127.0.0.1", TCP_PORT);
-        }
-        String address = iggyServer.getHost();
-        Integer port = iggyServer.getMappedPort(TCP_PORT);
-        return new IggyTcpClient(address, port);
+    static IggyTcpClient create(String host, int port) {
+        var client = Iggy.tcpClientBuilder().blocking().host(host).port(port).build();
+        client.connect();
+        return client;
     }
 }
