@@ -30,7 +30,7 @@ class UsersTcpClientTest extends UsersClientBaseTest {
 
     @Override
     protected IggyBaseClient getClient() {
-        return TcpClientFactory.create(iggyServer);
+        return TcpClientFactory.create(serverHost(), serverTcpPort());
     }
 
     @BeforeEach
