@@ -22,12 +22,12 @@ rootProject.name = "iggy-java-sdk"
 include("iggy")
 project(":iggy").projectDir = file("java-sdk")
 
-include("iggy-example")
-project(":iggy-example").projectDir = file("examples")
-
 // External processors - Stream processing integrations
 include("iggy-connector-library")
 project(":iggy-connector-library").projectDir = file("external-processors/iggy-connector-flink/iggy-connector-library")
 
 include("iggy-flink-examples")
 project(":iggy-flink-examples").projectDir = file("external-processors/iggy-connector-flink/iggy-flink-examples")
+
+include("iggy-connector-pinot")
+project(":iggy-connector-pinot").projectDir = file("external-processors/iggy-connector-pinot")
